import csv
import json
import requests
import pandas as pd
from .film import Film
from .common import wikipedia
from bs4 import BeautifulSoup


class AFITop100:
    """Class to obtain the AFI Top 100 Films of all time"""

    def __init__(self, **kwargs):
        self.wikipedia = wikipedia.Wikipedia()
        self.afi_list = list()

    def get_afi_list_by_year(self, year: int) -> list:
        """
        Given a valid list year, Return a list of Films objects sorted by AFI Ranking (Ascending)
        Valid list years as of Jan 2021 are 1998 and 2007
        """
        afi_list_by_year = list()
        for film in self.afi_list:
            if film[f"afi_rank_{year}"] is not None:
                afi_list_by_year.append(film)
        return sorted(afi_list_by_year, key=lambda k: k[f"afi_rank_{year}"])

    def scrape_afi_list(self, wiki_title="AFI's_100_Years...100_Movies", section="2") -> None:
        """
        Screen scrape the AFI top 100 list from the wikipedia page and store the data in this class instance
        """
        wiki_json = self.wikipedia.get_wikipedia_page(wiki_title, section=section)
        soup = BeautifulSoup(wiki_json["parse"]["text"], "html.parser")
        table = soup.find("table", attrs={"class": "wikitable"})
        for row in table.findAll("tr"):
            cells = row.findAll("td")
            if len(cells) == 6:
                film = Film(
                    cells[0].text.strip(),  # film title
                    int(cells[1].text.strip()),  # year will always be an integer
                    cells[2].text.strip(),  # director's name
                    cells[3].text.strip(),  # 1998 rank might be a '-' and not an integer so pass as a str
                    cells[4].text.strip(),  # 2007 rank might be a '-' and not an integer so pass as a str
                    None,
                )
                film.rank_delta = self.calculate_rank_movement(film)
                self.afi_list.append(film)

    def calculate_rank_movement(self, film: Film, afi_year_1=1998, afi_year_2=2007) -> int or None:
        """Return the rank_delta in the given film from year to year"""
        year_1 = getattr(film, f"afi_rank_{afi_year_1}")
        year_2 = getattr(film, f"afi_rank_{afi_year_2}")
        if year_1 is not None and year_2 is not None:
            return year_1 - year_2

    def get_film_by_title(self, title: str) -> Film or None:
        """Return a film object when searching for a fiulm by name (Case insensitive)"""
        for film_obj in self.afi_list:
            if film_obj.title.lower() == title.lower():
                return film_obj

    def get_afi_list_json(self, year=0) -> str:
        """Return a json string representation of the AFI TOP 100 list"""
        if year == 0:
            return json.dumps([film.__dict__ for film in self.afi_list], indent=2)
        else:
            year_list = self.get_afi_list_by_year(year)
            return json.dumps([film.__dict__ for film in year_list], indent=2)

    def get_afi_list_csv(self, year=0) -> str:
        if year == 0:
            return pd.DataFrame(self.afi_list).to_csv(index=False, float_format="%.f", quoting=csv.QUOTE_ALL)
        else:
            year_list = self.get_afi_list_by_year(year)
            return pd.DataFrame(year_list).to_csv(index=False, float_format="%.f", quoting=csv.QUOTE_ALL)



from unittest import TestCase
from afitop100.afi import AFITop100
import json

FILM_TITLE = "Citizen Kane"


class TestAFITop100(TestCase):
    def setUp(self):
        self.afitop100 = AFITop100()
        self.afitop100.scrape_afi_list()

    def test_get_top_100_of_all_time(self):
        self.assertGreater(len(self.afitop100.afi_list), 99)

    def test_get_afi_list_by_year(self):
        afi_1998 = self.afitop100.get_afi_list_by_year(1998)
        afi_2007 = self.afitop100.get_afi_list_by_year(2007)
        self.assertEqual(len(afi_1998), 100)
        self.assertEqual(len(afi_2007), 100)

    def test_get_rank_movement(self):
        # We need to choose films that are in both the 1998 list and the 2007 list to test rank movement
        for film_obj in self.afitop100.afi_list:
            if film_obj.afi_rank_1998 is not None and film_obj.afi_rank_2007 is not None:
                self.assertIsNotNone(self.afitop100.calculate_rank_movement(film_obj))

    def test_get_film_by_title(self):
        film = self.afitop100.get_film_by_title(FILM_TITLE)
        self.assertEqual(film.title.lower(), FILM_TITLE.lower())

    def test_get_top_100_of_all_time_json(self):
        json_list = json.loads(self.afitop100.get_afi_list_json())
        self.assertGreaterEqual(len(json_list), 99)

    def test_get_afi_list_by_year_json(self):
        json_list = json.loads(self.afitop100.get_afi_list_json(year=2007))
        self.assertGreaterEqual(len(json_list), 99)

    def test_get_afi_list_by_year_csv(self):
        csv_list = self.afitop100.get_afi_list_csv(year=2007).splitlines()
        self.assertGreaterEqual(len(csv_list), 99)



import sys
import logging
import argparse
from afitop100 import AFITop100

logger = logging.getLogger("afitop100")


def main():
    """🧙‍✨ where the magic happens ✨‍🧙"""

    parser = argparse.ArgumentParser(
        prog="afitop100", usage="%%(prog) [options]", description="Get the AFI Top 100 List(s)"
    )
    parser.add_argument("-o", "--out", type=str, default="json", help="Output format -o json or -o csv")
    parser.add_argument(
        "-y", "--year", type=int, default=0, help="Limit the output to a specific year -y 1998 or -y 2007"
    )
    arguments = parser.parse_args()

    afi(arguments)


def afi(arguments):
    """Get the AFI list from wikipedia and present in the format requested in the cli parameters"""
    afi_100 = AFITop100()
    afi_100.scrape_afi_list()

    if arguments.out.lower().strip() == "json":
        print(afi_100.get_afi_list_json(arguments.year))
    elif arguments.out.lower().strip() == "csv":
        print(afi_100.get_afi_list_csv(arguments.year))


if __name__ == "__main__":
    sys.exit(main())


"""AFI Top 100 Liste Generator for Python
>>> from afitop100 import AFITop100
>>> afi = AFITop100()
>>> afi.scrape_afi_list()
>>> print(afi.get_afi_list_json())
"""

__version__ = "1.0.1"

from .afi import AFITop100



from dataclasses import dataclass


@dataclass
class Film:
    """Class to hold film data. AFI List is not updated often, but as new rankings come out, we will need to add new
    attributes for each year. We may want to abstract this to make things scalable."""

    title: str
    release_year: int
    director: str
    afi_rank_1998: int
    afi_rank_2007: int
    rank_delta: int

    def __setattr__(self, name, value) -> None:
        """The year-rank is not assured to be an integer, so do some input checking and convert as needed"""
        if name == "afi_rank_1998" or name == "afi_rank_2007":
            if self.is_int_str(value):
                self.__dict__[name] = int(value)
            else:
                self.__dict__[name] = None
        else:
            self.__dict__[name] = value

    def __getitem__(self, key):
        """This makes the Film class subscriptable"""
        return getattr(self, key)

    def is_int_str(self, s: str) -> bool:
        """Determine if the given string can be represented as an integer"""
        try:
            int(s)
            return True
        except ValueError:
            return False



import requests

BASE_URL = "https://en.wikipedia.org"


class Wikipedia:
    """
    Retrieve a wikipedia page or section and return it in json format
    """

    def __init__(self, **kwargs: dict):
        if kwargs.get("url"):
            self.base_url = kwargs["url"]
        else:
            self.base_url = BASE_URL

    def get_wikipedia_page(self, page: str, **kwargs: dict) -> dict:
        """Given a wikipedia page, return a json represnetation of the page and the section if one is given."""

        headers = (
            {"User-Agent": kwargs["user_agent"]} if kwargs.get("user_agent") else {"User-Agent": "afitop100-restclient"}
        )

        querystring = {"action": "parse", "format": "json", "page": page, "prop": "text", "formatversion": "2"}

        if kwargs.get("section"):
            querystring["section"] = kwargs["section"]

        r = requests.request("GET", f"{self.base_url}/w/api.php", headers=headers, params=querystring)
        return r.json()



import setuptools
import codecs
import os


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


setuptools.setup(
    name="afitop100",
    packages=setuptools.find_packages(),
    version=get_version("afitop100/__init__.py"),
    license="GPL 3.0 https://www.gnu.org/licenses/gpl-3.0.txt",
    description="Get the AFI Top 100 movies of all time in json or csv",
    author="Aaron K. Hackney",
    author_email="aaron_309@yahoo.com",
    url="https://github.com/aaronhackney/afitop100",
    download_url="https://github.com/aaronhackney/afitop100/archive/v1.0.1.tar.gz",
    keywords=["afi", "top 100", "films", "movies", "all time", "american", "film", "institute"],
    install_requires=["beautifulsoup4", "requests", "pandas"],
    entry_points={"console_scripts": ["afitop100 = afitop100.__main__:main"]},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)